/**
 Copyright (c) 2012 Wireless Designs, LLC

 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
 "Software"), to deal in the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.hudak.zoltan.temperaturebeacon;

import android.app.Activity;
import android.app.ListActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import java.util.List;
import java.util.HashMap;
import android.widget.ArrayAdapter;
import android.graphics.Color;

public class MainActivity extends ListActivity {
    //    private LeDeviceListAdapter mLeDeviceListAdapter;
    private BluetoothAdapter mBluetoothAdapter;
    private boolean mScanning;
    private Handler mHandler;

    private static final int REQUEST_ENABLE_BT = 1;
    // Stops scanning after 10 seconds.
//    private static final long SCAN_PERIOD = 10000;
    /* Collect unique devices discovered, keyed by address */
    private HashMap<String, TemperatureBeacon> mBeacons;
    private BeaconAdapter mAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getActionBar().setTitle(R.string.title_devices);

        mHandler = new Handler();

        // Use this check to determine whether BLE is supported on the device.  Then you can
        // selectively disable BLE-related features.
        if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
            Toast.makeText(this, R.string.ble_not_supported, Toast.LENGTH_SHORT).show();
            finish();
        }

        // Initializes a Bluetooth adapter.  For API level 18 and above, get a reference to
        // BluetoothAdapter through BluetoothManager.
        final BluetoothManager bluetoothManager =  (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();

        // Checks if Bluetooth is supported on the device.
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, R.string.error_bluetooth_not_supported, Toast.LENGTH_SHORT).show();
            finish();
            return;
        }

        mBeacons = new HashMap<String, TemperatureBeacon>();
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        if(mAdapter.mCelsius) {
            menu.findItem(R.id.menu_celsius).setVisible(false);
            menu.findItem(R.id.menu_fahrenheit).setVisible(true);
        } else {
            menu.findItem(R.id.menu_celsius).setVisible(true);
            menu.findItem(R.id.menu_fahrenheit).setVisible(false);
        }
        if (!mScanning) {
            menu.findItem(R.id.menu_refresh).setActionView(null);
        } else {
           menu.findItem(R.id.menu_refresh).setActionView(R.layout.actionbar_indeterminate_progress);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_celsius:
                mAdapter.setCelsius(true);
                break;
            case R.id.menu_fahrenheit:
                mAdapter.setCelsius(false);
                break;
        }

        stopScan();
        mAdapter.clear();
        mHandler.removeCallbacks(mStopRunnable);
        mHandler.removeCallbacks(mStartRunnable);
        startScan();

        return true;
    }

    protected void onResume() {
        super.onResume();

        // Ensures Bluetooth is enabled on the device.  If Bluetooth is not currently enabled,
        // fire an intent to display a dialog asking the user to grant permission to enable it.
        if (!mBluetoothAdapter.isEnabled()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        }

        // Initializes list view adapter.
        mAdapter = new BeaconAdapter(this);
        setListAdapter(mAdapter);
//        scanLeDevice(true);
        startScan();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        // User chose not to enable Bluetooth.
        if (requestCode == REQUEST_ENABLE_BT && resultCode == Activity.RESULT_CANCELED) {
            finish();
            return;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    protected void onPause() {
        super.onPause();
//        scanLeDevice(false);
        stopScan();
        mAdapter.clear();
        mHandler.removeCallbacks(mStopRunnable);
        mHandler.removeCallbacks(mStartRunnable);
    }

//    private void scanLeDevice(final boolean enable) {
//        if (enable) {
//            // Stops scanning after a pre-defined scan period.
//            mHandler.postDelayed(new Runnable() {
//                @Override
//                public void run() {
//                    mScanning = false;
//                    mBluetoothAdapter.stopLeScan(mLeScanCallback);
//                    invalidateOptionsMenu();
//                }
//            }, SCAN_PERIOD);
//
//            mScanning = true;
//            mBluetoothAdapter.startLeScan(mLeScanCallback);
//        } else {
//            mScanning = false;
//            mBluetoothAdapter.stopLeScan(mLeScanCallback);
//        }
//        invalidateOptionsMenu();
//    }

    private Runnable mStopRunnable = new Runnable() {
        @Override
        public void run() {
            stopScan();
        }
    };
    private Runnable mStartRunnable = new Runnable() {
        @Override
        public void run() {
            startScan();
        }
    };

    private void startScan() {
        //Scan for devices advertising the thermometer service
        mScanning = true;
        mBluetoothAdapter.startLeScan(mLeScanCallback);
        //setProgressBarIndeterminateVisibility(true);
        invalidateOptionsMenu();

        mHandler.postDelayed(mStopRunnable, 5000);
    }

    private void stopScan() {
        mScanning = false;
        mBluetoothAdapter.stopLeScan(mLeScanCallback);
        //setProgressBarIndeterminateVisibility(false);
        invalidateOptionsMenu();

        mHandler.postDelayed(mStartRunnable, 5000);
    }


    // Device scan callback.
    private BluetoothAdapter.LeScanCallback mLeScanCallback =
            new BluetoothAdapter.LeScanCallback() {

                @Override
                public void onLeScan(final BluetoothDevice device, final int rssi, final byte[] scanRecord) {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            List<AdRecord> records = AdRecord.parseScanRecord(scanRecord);
                            TemperatureBeacon beacon = new TemperatureBeacon(records, device.getAddress(), rssi);
                            mBeacons.put(beacon.getName(), beacon);
                            mAdapter.setNotifyOnChange(false);
                            mAdapter.clear();
                            mAdapter.addAll(mBeacons.values());
                            mAdapter.notifyDataSetChanged();
                        }
                    });
                }
            };


    /*
 * A custom adapter implementation that displays the TemperatureBeacon
 * element data in columns, and also varies the text color of each row
 * by the temperature values of the beacon
 */
    private static class BeaconAdapter extends ArrayAdapter<TemperatureBeacon> {

        private boolean mCelsius = true;

        public BeaconAdapter(Context context) {
            super(context, 0);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            if (convertView == null)
                convertView = LayoutInflater.from(getContext()).inflate(R.layout.activity_main, parent, false);

            TemperatureBeacon beacon = getItem(position);
            //Set color based on temperature
            final int textColor = getTemperatureColor(beacon.getCurrentTemp());

            TextView nameView = (TextView) convertView.findViewById(R.id.text_name);
            nameView.setText(beacon.getName());
            nameView.setTextColor(textColor);

            TextView tempView = (TextView) convertView.findViewById(R.id.text_temperature);
            if(mCelsius) {
                tempView.setText(String.format("%.1f\u00B0C", beacon.getCurrentTemp()));
            } else {
                tempView.setText(String.format("%.1f\u00B0F", beacon.getCurrentTemp() * 9/5 + 32));
            }
            tempView.setTextColor(textColor);

//            TextView addressView = (TextView) convertView.findViewById(R.id.text_address);
//            addressView.setText(beacon.getAddress());
//            addressView.setTextColor(textColor);
//
//            TextView rssiView = (TextView) convertView.findViewById(R.id.text_rssi);
//            rssiView.setText(String.format("%ddBm", beacon.getSignal()));
//            rssiView.setTextColor(textColor);

            return convertView;
        }

        public void setCelsius(boolean celsius) {
            mCelsius = celsius;
        }

        private int getTemperatureColor(float temperature) {
            //Color range from 0 - 40 degC
            float clipped = Math.max(0f, Math.min(40f, temperature));

            float scaled = ((40f - clipped) / 40f) * 255f;
            int blue = Math.round(scaled);
            int red = 255 - blue;

            return Color.rgb(red, 0, blue);
        }
    }
}
